function [x,Rx,yp,Ryp,xp,Rxp] = Kalman(x,y,u,M,N,F,A,B,G,Rw,Rv,Rx)
  // Kalman filter for state estimtion with the model
  //                             x = M*x + N*u + F + w
  //                             yp = A*x + B*u + G + e
  //                             for missing parameters set empty - []
  // x     state
  // Rx    state estimate covariance matrix
  // yp    output prediction
  // Ryp    covariance of output prediction
  // y     output
  // u     input
  // M,N,F state model parameters
  // A,B,G output model parameters
  // Rw    state model covariance
  // Rv    output model covariance

  nx=size(M,1);                                                         // 1
  ny=size(A,1);                                                         // 2
  nu=length(u);                                                         // 3
  if isempty(F) then F=zeros(nx,1);end,                                 // 4
  if isempty(G) then G=zeros(ny,1);end,                                 // 5
  if isempty(N) then N=zeros(nx,nu);end,                                // 6
  if isempty(B) then B=zeros(ny,nu);end,                                // 7
                                                                        // 8
  xp=M*x+N*u+F;                    // time update of the state          // 9
  Rxp=Rw+M*Rx*M';                  // time updt. of state covariance    // 10
                                                                        // 11
  yp=A*xp+B*u+G;                   // output prediction                 // 12
  Ryp=Rv+A*Rx*A';                  // noise covariance update           // 13
  Rx=Rxp-Rxp*A'*inv(Ryp)*A*Rxp;    // state est. covariance update      // 14
  ey=y-yp;                         // prediction error                  // 15
  KG=Rx*A'*inv(Rv);                // Kalman gain                       // 16
  x=xp+KG*ey;                      // data update of the state          // 17
endfunction                                                             
