// T11simReg3.sce
// SECOND ORDER REGRESSION MODEL AND LS ESTIMATION
// Experiments
// - change parameters of the model
// - change the input signal
// - try to increase the model order to 3
// -------------------------------------------------------------------
exec("ScIntro.sce",-1), mode(0)

// PARAMETERS                                                           // 1
nd=100;                       // length of data                         // 2
aS=[.4 .2];                   // parameters at y                        // 3
bS=[1 .2 -.5];                // parameters at u                        // 4
kS=5;                         // constant (model absolute term)         // 5
sS=.1;                        // noise std                              // 6
                                                                        // 7
y(1)=1; y(2)=3;               // initial conditions for output          // 8
u=signal(nd,1,.1);            // input                                  // 9
                                                                        // 10
// TIME LOOP                                                            // 11
thS=[aS bS kS]';              // vector of parameters                   // 12
for t=3:nd                                                              // 13
  ps=[y(t-1) y(t-2) u(t) u(t-1) u(t-2) 1]';  // regression vector       // 14
  y(t)=thS'*ps+sS*rand(1,1,'n');             // simulation              // 15
  Y(t)=y(t);                                                            // 16
  X(t,:)=ps';                                                           // 17
end                                                                     // 18
thE=inv(X'*X)*X'*Y;                                                     // 19
                                                                        // 20
// RESULTS OF THE SIMULATION                                            // 21
set(gcf(),"position",[700 100 600 500])                                 // 22
subplot(211),plot(1:nd,u),title('Input')                                // 23
subplot(212),plot(1:nd,y),title('Output')                               // 24
disp('The simulated and estimated parameters are',[thS,thE])            // 25
