// T14simCat.sce
// SIMULATION OF DISCRETE MODEL
//  (multinomial model - controlled coin with memmory)
//         f( y(t) | u(t),y(t-1) ),  y,u=1,2
// Experiments
// - set the parameters to obtain a deterministic model
// - try to extend the model to f(y(t)|u(t),y(t-1),u(t-1)) 
//   and values 1,2,3.
// -------------------------------------------------------------------
exec("ScIntro.sce",-1), mode(0)

nd=500;                                       // number of steps        // 1
// PARAMETERS                                                           // 2
//y(t)=1 =2  //  u(t) y(t-1)                                            // 3
// ---------------------------                                          // 4
// u(t)    1  1  2  2                        // model                   // 5
// y(t-1)  1  2  1  2                                                   // 6
thS=     [.2 .6 .9 .7    // y(t)=1                                      // 7
          .8 .4 .1 .3];  // y(t)=2                                      // 8
y(1)=1; u(1)=0;                              // initial conditions      // 9
S=zeros(2,4);                                // initial statistics      // 10
                                                                        // 11
// TIME LOOP                                                            // 12
for t=2:nd                                                              // 13
  // simulation                                                         // 14
  u(t)=sum(cumsum([.3 .7])<rand(1,1,'u'))+1; // control                 // 15
  k=2*(u(t)-1)+y(t-1);                       // column in the model     // 16
  y(t)=sum(cumsum(thS(:,k))<rand(1,1,'u'))+1;// output                  // 17
  // estimation                                                         // 18
  S(y(t),k)=S(y(t),k)+1;                                                // 19
end                                                                     // 20
thE=fnorm(S,1);                                                         // 21
                                                                        // 22
// RESULTS                                                              // 23
subplot(211),plot(1:nd,u,'g:.')                                         // 24
set(gcf(),"position",[700 100 600 500])                                 // 25
title('Input')                                                          // 26
set(gca(),'data_bounds',[0 nd+1 .9 2.1])                                // 27
subplot(212),plot(1:nd,y,'b:.')                                         // 28
title('Output')                                                         // 29
set(gca(),'data_bounds',[0 nd+1 .9 2.1])                                // 30
                                                                        // 31
disp('Model parameters',thS)                                            // 32
disp('Estimat od model parameters',thE)                                 // 33
