// T23preReg1.sce
// K-STEP POINT PREDICTION WITH REGRESSION MODEL (KNOWN PARAMETERS)
// Experiments
// Change: - np = number of steps of prediction
//         - r  = noise variance
//         - th = model parametrs
//         - u  = input signal
// Shift of reression vector
// ps(t)=  [u(t) y(t-1) u(t-1) y(t-2) u(t-2) 1]     at time t
//          |----------------|
// ps(t+1)=[u(t+1) y(t) u(t) y(t-1) u(t-1) 1]       at time t+1
//                      |----------------|
// -------------------------------------------------------------------
exec("ScIntro.sce",-1), mode(0)

// PARAMETERS                                                           // 1
nd=100;                            // number of data                    // 2
np=5;                              // length of prediction (np>=1)      // 3
//   b0 a1 b1 a2  b2 k                                                  // 4
thS=[1 .4 -.3 -.5 .1 1]';          // regression coefficients           // 5
rS=.02;                            // noise variance                    // 6
u=signal(nd,1);                    // input                             // 7
y(1)=1; y(2)=-1;                   // prior data                        // 8
                                                                        // 9
// TIME LOOP                                                            // 10
for t=3:(nd-np)                    // time loop (on-line tasks)         // 11
  // prediction                                                         // 12
  ps=[u(t) y(t-1) u(t-1) y(t-2) u(t-2) 1]'; // first reg. vec           // 13
  yy=ps'*thS;                      // zero prediction for time = t      // 14
  for j=1:np                       // predictions for t+1,...,t+np      // 15
    ps=[u(t+j); yy; ps(1:$-3); 1]; // reg.vecs with predicted outputs   // 16
    yy=ps'*thS;                    // new prediction (partial)          // 17
  end                                                                   // 18
  yp(t+np)=yy;                     // final prediction for time t+np    // 19
                                                                        // 20
  // simulation                                                         // 21
  ps=[u(t) y(t-1) u(t-1) y(t-2) u(t-2) 1]'; // reg. vector for sim.     // 22
  y(t)=ps'*thS+sqrt(rS)*rand(1,1,'n');      // output generation        // 23
end                                                                     // 24
                                                                        // 25
// RESULTS                                                              // 26
s=(np+3):(nd-np);                                                       // 27
scf(1);                                                                 // 28
plot(s,y(s),'.:',s,yp(s),'rx')                                          // 29
set(gca(),"data_bounds",[1 nd -3 5])                                    // 30
legend('output','prediction');                                          // 31
title(string(np)+'-steps ahead prediction')                             // 32
                                                                        // 33
RPE=variance(y(s)-yp(s))/variance(y)    // relative prediction error    // 34
