// T25preReg3.sce
// K-STEP PREDICTION WITH REGRESSION MODEL (REAL DATA) 
// - intensities from Strahov tunnel are used
// Experiments
// Change: - np  = number of steps of prediction
//         - ord = order of the model for estimation 
// Result: - visual comparison of yt and yp
//         - RPE = relative prediction error
// -------------------------------------------------------------------
exec("ScIntro.sce",-1), mode(0)

// PARAMETERS                                                           // 1
np=5;                              // length of prediction (np>=1)      // 2
ord=5;                             // order of the estimated model      // 3
// data selection                                                       // 4
k=3;                               // which day is selected             // 5
nz=50;                             // beginning of the day              // 6
nd=288*k;                          // length of the whole one day data  // 7
                                                                        // 8
// DATA                                                                 // 9
dtAll=csvRead('STRAHOV.csv',';');  // load data                         // 10
dt=dtAll((k-1)*288+(nz+1:nd),1);                                        // 11
nth=ord+2;                         // size of V                         // 12
V=1e-8*eye(nth,nth);               // initial information matrix        // 13
thE=rand(nth-1,1,'n');             // prior parametrs                   // 14
yt=dt(1:ord);                      // prior data                        // 15
                                                                        // 16
// TIME LOOP                                                            // 17
for t=ord+1:(nd-np-nz)             // time loop (on-line tasks)         // 18
//  prediction                                                          // 19
  ps=[yt(t-1:-1:t-ord); 1];        // regression vector                 // 20
  yy=ps'*thE;                      // first prediction at t+1           // 21
  for j=1:np                       // predictions for t+2,...,t+np      // 22
    ps=[yy; ps(1:(ord-1)); 1];     // reg. vector (no control)          // 23
    yy=ps'*thE;                    // new prediction (partial)          // 24
  end                                                                   // 25
  yp(t+np,1)=yy;                   // final prediction for time t+np    // 26
                                                                        // 27
  // data measurement (as if)                                           // 28
  yt(t)=dt(t);                     // measuring of output               // 29
                                                                        // 30
  // estimation                                                         // 31
  psi=[yt(t:-1:t-ord); 1];                   // reg. vector             // 32
  V=V+psi*psi';                              // updt of inf. matrix     // 33
  Vy=V(1,1); Vyp=V(2:$,1); Vp=V(2:$,2:$);    // division of inf. matrix // 34
  thE(:,1)=inv(Vp+1e-8*eye(Vp))*Vyp;         // est. of reg. coeff      // 35
  r(t)=(Vy-Vyp'*inv(Vp+1e-8*eye(Vp))*Vyp)/t; // est. of noise variance  // 36
  Et(:,t)=thE;                               // stor est. parameters    // 37
end                                                                     // 38
                                                                        // 39
// Results                                                              // 40
// evolution of parametrs                                               // 41
set(scf(1),'position',[100 100 1200 400]);                              // 42
subplot(121),plot(Et')                                                  // 43
set(gca(),"data_bounds",[0 nd+1 -1 5])                                  // 44
title('Evolution of estimated parameters')                              // 45
legend('b0E','a1E','b1E','a2E','b2E','kE');                             // 46
// comparison of yt and yp                                              // 47
subplot(122)                                                            // 48
s=1:length(yt);                                                         // 49
plot(s,yt(s),'.:',s,yp(s),'rx')                                         // 50
set(gca(),"data_bounds",[1 nd -4 max(yt)+5])                            // 51
legend('output','prediction');                                          // 52
title([string(np),'- step ahead prediction'])                           // 53
// Relative Prediction Error                                            // 54
RPE=variance(yt(s)-yp(s))/variance(yt(s))                               // 55
