// T61mixExpNor1.sce
// MIXTURE ESTIMATION (predictive normal)
// - scalar case of T62mixExpNor
// - dynamic normal componens
// - dynamic pointer
// - weight for initial information
// - scalar variables
//   y(t)=a1.y(t-1)+a2.y(t-2)+b.u(t)+e(t)
// Experiments: Change
// - simulated parameters thS 
//   (including model order for both simulation and estimation)
// - initial parameters thI (through the statistics V, ka)
// - the course of input signal (smooth signal is bad for estimation)
// - type of prediction (either point or generated), 
// - strength of initial information (through the statistics ka), 
// - parameters of pointer model alS (the more is diagonal-like, 
//   the longer are inervals between svitching)
// -------------------------------------------------------------------
exec("ScIntro.sce",-1), 
getd(), mode(0)

nd=500;                                                                 // 1
ni=1;                                   // weight for initial inf.      // 2
// PARAMETERS                                                           // 3
aS=list();                                                              // 4
aS(1)=[.6 -.5]; bS(1)=.8; kS(1)=-1;     // regression coefficients      // 5
aS(2)=[.2 .4]; bS(2)=-1.8; kS(2)=1;     // regression coefficients      // 6
sd=.1;                                  // common and known noise std.  // 7
nc=length(aS);                          // number of components         // 8
alS=[.8 .1                              // parameter of pointer model   // 9
     .2 .9];                                                            // 10
y(1)=2; y(2)=-1; cS=ones(1,nc);         // initial conditions           // 11
u=signal(nd,1);                         // definition of input signal   // 12
                                                                        // 13
// INITIALIZATION                                                       // 14
V=list(); thI=list(); thE=list();                                       // 15
ka=ones(1,nc)*ni;                        // initial counter             // 16    
for j=1:nc                                                              // 17
  thI(j)=[aS(j) bS(j) kS(j)]+1*rand(1,4); // initial parameters         // 18
  V(j)=eye(5,5);                        // initial sum statistics       // 19 
  V(j)(1,2:5)=thI(j);                                                   // 20
  V(j)(2:5,1)=thI(j)';                                                  // 21
  V(j)=V(j)*ni;                                                         // 22
  thE(j)=v2thN(V(j)/ka(j),1);           //initial point estimates       // 23   
  c(j).th(:,2)=thE(j)(:);               // remember for plot            // 24
end                                                                     // 25
ga=eye(nc,nc);                          // statistics for pointer model // 26
alE=fnorm(ga,2);                        // point estimate               // 27
ws=fnorm(ones(nc,1));                   // old component weights        // 28
                                                                        // 29
// TIME LOOP                                                            // 30
for t=3:nd                                                              // 31
  // prediction                                                         // 32
  ypp=0;                                                                // 33
  fc=fnorm(alE*ws);                     // f(c(t-1)|d(t))               // 34
  ps=[y(t-1) y(t-2) u(t) 1]';           // regression vector            // 35
  for j=1:nc                                                            // 36
    select 1        // type of prediction: 1-point, 2-generated;        // 37
    case 1, ypp=ypp+fc(j)*thE(j)'*ps;                                   // 38  
    case 2, ypp=ypp+fc(j)*(thE(j)'*ps+sd*randn());                      // 39
    end                                                                 // 40
  end                                                                   // 41
  yp(:,t)=ypp;                          // prediction at time t         // 42
                                                                        // 43
  // simulation                                                         // 44
  jS=sampCat(alS,cS(t-1));              // pointer value                // 45
  cS(t)=jS;                             // remember                     // 46
  y(t)=[aS(jS) bS(jS) kS(jS)]*[y(t-1) y(t-2) u(t) 1]'+sd*randn();       // 47
                                        // output generation            // 48
                                                                        // 49
  // estimation                                                         // 50
  for j=1:nc                                                            // 51
    [nill,qp(j)]=GaussN(y(t),thE(j)'*ps,sd); // component proximity     // 52                  
  end                                                                   // 53
  q=exp(qp-max(qp)).*(alE*ws);          // overall proximity            // 54
  w=q/sum(q);                           // component weights            // 55             
  wt(:,t)=w;                            // remember                     // 56
  for j=1:nc                                                            // 57
    Ps=[y(t) y(t-1) y(t-2) u(t)1]';     // extended regression vector   // 58
    V(j)=V(j)+w(j)*Ps*Ps';              // update of sum statistics     // 59   
    ka(j)=ka(j)+w(j);                   // update of counter            // 60  
    thE(j)=v2thN(V(j)/ka(j),1);         // pt estimate of components    // 61      
    c(j).th(:,t)=thE(j)(:);             // remember for plot            // 62   
  end                                                                   // 63
  ga=ga+w*ws';                          // stat. update of poiner mod.  // 64
  alE=fnorm(ga,2);                      // pt estimate of pointer model // 65
  ws=w;                                 // w -> old w                   // 66
  wt(:,t)=w;                            // remember                     // 67
end                                                                     // 68
                                                                        // 69
// RESULTS                                                              // 70
set(scf(1),'position',[600 10 1200 400])                                // 71
for j=1:nc                                                              // 72
  subplot(1,nc,j)                                                       // 73
  plot(c(j).th(:,2:$)')                                                 // 74
  title('Evolution of estimated parametrs: comp. '+string(j))           // 75                
end                                                                     // 76
                                                                        // 77
set(scf(2),'position',[900 500 600 400])  // plot data and prediction   // 78     
plot(1:nd,y,'ob',1:nd,yp,'xr','markersize',8)                           // 79                     
title 'Data (b) and prediction (r)'                                     // 80  
                                                                        // 81
[nill,cp]=max(wt,'r');                    // accuracy of classification // 82
disp 'Accuracy of classification'                                       // 83
ACC=acc(cS,cp)                                                          // 84
                                                                        // 85
disp 'Relative prediction error'          // accuracy of prediction     // 86
RPE=rpe(y(:),yp(:))                                                     // 87
                                                                        // 88
disp 'Simulated and estimated parameters - comp. 1'                     // 89
disp([aS(1) bS(1) kS(1);thE(1)'])         // parameters of first comp.  // 90
                                                                        // 91
disp 'Simulated and estimated parameters - comp. 2'                     // 92
disp([aS(2) bS(2) kS(2);thE(2)'])         // parameters of second comp. // 93
                                                                       
