// T62mixExpNor.sce
// MIXTURE ESTIMATION (predictive normal)
// - dynamic normal componens
// - weight for initial information
// - multinomial y and v
//   |y1|    |a11 a12||y1|   |b11 b12 b13||v1|
//   |  | =  |       ||  | + |           ||v2| + e
//   |y2|    |a21 a22||y2|   |b21 b22 b23||v3|
//    new             old                 actual
// Experiments
// - change simulated expectations thS
// - change initial expectations thI (through the statistics S, ka)
// -------------------------------------------------------------------
exec("ScIntro.sce",-1), 
getd(), mode(0)

nd=500;                                                                 // 1
ni=1;                                   // weight for initial info.     // 2
// PARAMETERS                                                           // 3
sd=.01;                                 // std of the noise             // 4
aS=list(); bS=list(); kS=list();                                        // 5
aS(1)=[.7 -.2 .3 -.2                    //regression coefficients       // 6
       .1 .5  .1 .4];                                                   // 7
aS(2)=[.2 .4  .6 .2                                                     // 8
      -.1 .7  .2 .1];                                                   // 9
aS(3)=[.3 -.3 .4 .1                                                     // 10
      -.2 .6  .1 .4];                                                   // 11
bS(1)=[1; 3];                                                           // 12
bS(2)=[8; 4];                                                           // 13
bS(3)=[6; 5];                                                           // 14
kS(1)=[1; -1];                                                          // 15
kS(2)=[-1; 1];                                                          // 16
kS(3)=[1; 1];                                                           // 17
alS=[.7  2 .1                                                           // 18
     .2 .5 .3                                                           // 19
     .1 .3 .6];                         // parameters of pointer model  // 20
alS=[.9  .05 .05                                                        // 21
     .05 .9  .05                                                        // 22
     .05 .05 .9];                       // parameters of pointer model  // 23
y=zeros(2,4);                                                           // 24
u=signal(nd,1);                         // definition of input signal   // 25
nc=length(aS);                          // number of components         // 26
                                                                        // 27
// INITIALIZATION                                                       // 28
V=list(); thI=list(); thE=list();                                       // 29
ka=[1 1 1]*ni;                          // initial counter              // 30   
for j=1:nc                                                              // 31
  thI(j)=[aS(j) bS(j) kS(j)]+.1*rand(2,6); // initial comp. parameters  // 32
  V(j)=eye(8,8);                        // initial sum statistics       // 33 
  V(j)(1:2,3:$)=thI(j);                                                 // 34
  V(j)(3:$,1:2)=thI(j)';                                                // 35
  V(j)=V(j)*ni;                                                         // 36
  thE(j)=v2thN(V(j)/ka(j),2);           // initial point estimates      // 37    
  c(j).th=thE(j)(:);                    // remember                     // 38
end                                                                     // 39
ga=eye(nc,nc);                          // statistics for pointer model // 40
alE=fnorm(ga,2);                        // point estimate               // 41
ws=fnorm(ones(nc,1));                                                   // 42
                                                                        // 43
// TIME LOOP                                                            // 44
for t=3:nd                                                              // 45
  // prediction                                                         // 46
  ypp=0;                                                                // 47
  fc=fnorm(alE*ws);                     // f(c(t-1)|d(t))               // 48
  ps=[y(:,t-1)' y(:,t-2)' u(t) 1]';                                     // 49
  for j=1:nc                                                            // 50
    select 2        // type of prediction: 1-point, 2-generated;        // 51
    case 1, ypp=ypp+fc(j)*thE(j)'*ps;                                   // 52
    case 2, ypp=ypp+fc(j)*(thE(j)'*ps+sd*randn(2,1));                   // 53
    end                                                                 // 54
  end                                                                   // 55
  yp(:,t)=ypp;                          // remember value of prediction // 56                 
                                                                        // 57
  // simulation                                                         // 58
  jS=sampCat(alS);                      // pointer value                // 59       
  cS(t)=jS;                                                             // 60
  par=[aS(jS) bS(jS)];                  // parameters                   // 61
  reg=[y(:,t-1)' y(:,t-2)' u(t)'];      // data                         // 62
  y(:,t)=par*reg'+sd*randn(2,1);        // output generation            // 63             
                                                                        // 64
  // estimation                                                         // 65
  for j=1:nc                                                            // 66
    ps=[y(:,t-1)' y(:,t-2)' u(t-1)' 1]';                                // 67
    [nill,qp(j)]=GaussN(y(:,t),thE(j)'*ps,sd*eye(2,2));                 // 68
                                        // component proximity          // 69             
  end                                                                   // 70
  q=exp(qp-max(qp)).*(alE*ws);          // full proximity               // 71
  w=q/sum(q);                           // weights                      // 72   
  for j=1:nc                                                            // 73
    Ps=[y(:,t)' y(:,t-1)' y(:,t-2)' u(t)' 1]';                          // 74
    V(j)=V(j)+w(j)*Ps*Ps';              // update of sum statistics     // 75   
    ka(j)=ka(j)+w(j);                   // update of counter            // 76   
    thE(j)=v2thN(V(j)/ka(j),2);         // point estimates              // 77   
    c(j).th(:,t)=thE(j)(:);             // remember for plot            // 78   
  end                                                                   // 79
                                                                        // 80
  ga=ga+w*ws';                          // stat. update of poiner mod.  // 81
  alE=fnorm(ga,2);                      // pt estimate of pointer model // 82
  ws=w;                                 // w -> old w                   // 83
  wt(:,t)=w;                            // remember                     // 84
end                                                                     // 85
                                                                        // 86
// RESULTS                                                              // 87
set(scf(1),'position',[600 10 1200 400])                                // 88
for j=1:nc                                                              // 89
  subplot(1,3,j)                                                        // 90
  plot(c(j).th')                                                        // 91
  title('Evolution of estimated parametrs: comp. '+string(j))           // 92                
end                                                                     // 93
                                                                        // 94
set(scf(2),'position',[900 500 600 400])  // plot data and prediction   // 95     
plot(1:nd,y,'ob',1:nd,yp,'xr','markersize',8)                           // 96                     
title 'Data (b) and prediction (r)'                                     // 97  
                                                                        // 98
[nill,cp]=max(wt,'r');                    // accuracy of classification // 99
disp 'Accuracy of classification'                                       // 100
ACC=acc(cS,cp)                                                          // 101
                                                                        // 102
disp 'Relative prediction error'          // accuracy of prediction     // 103
RPE=rpe(y(:),yp(:))                                                     // 104
