// T72mixMargD.sce
// Estimation of f(y,x1,x2) with different components
// - y generated from product of fy1*fy2
// kx(i)          nuber of values of var. (i)
// pI(i)(j)       initial parameters (var. i and comp. j)
// kI             initial counter (common to all)
// N(j).x(i).s    statistics (var. i and comp. j)
// N(j).x(i).k    counter (var. i and comp. j)
// C(j).x(i).p    parameters (var. i and comp. j)
// fy             maginal model for y
// f(j).x(i).c    local model f_j(x_i|y)
// fpi            weighted sum of local models
// Fyp            product of fpi
// ------------------------------------------
clc, clear, close(winsid()), mode(0)
getd functions

// DATA                                                               // D1 
nd=1000;                           // total number of dataset         // D2 
nL=800;                            // number of samples for learnig   // D3 
pS=list(); tH=list(); thY=list();                                     // D4 
// for x1        for x2                                               // D5 
tH(1)=[.2 .8];   tH(2)=[.3 .7];    // pointer parameters              // D6 
pS(1)=[.002 .994]; pS(2)=[.005  .991]; // binomial p                  // D7 
nS(1)=5+1;       nS(2)=5+1;        // binomial n                      // D8 
nX=length(nS);                     // numb. of x-variables            // D9 
ny=6;                                                                 // D10
thY(1)=genThs(ny,nS(1),.1);                                           // D11
thY(2)=genThs(ny,nS(2),.1);                                           // D12

// SIMULATION of the whole dataset (L + T)                            // S1 
for t=1:nd                                                            // S2 
  fY=ones(ny,1);                                                      // S3 
  for i=1:nX                                                          // S4 
    c(t,i)=sampCat(tH(i));                                            // S5 
    x(t,i)=sampBin1(pS(i)(c(t,i)),nS(i));                             // S6 
    fY=fY.*thY(i)(:,x(t,i));                                          // S7 
  end                                                                 // S8 
  fY=fnorm(fY);                                                       // S9 
  y(t)=sampCat(fY);                                                   // S10
end                                                                   // S11
                                                                      // S12
// Learning and Testing data                                          // S13
iL=samwr(nL,1,1:nd); iT=setdiff(1:nd,iL);                             // S14
xL=x(iL,:);          xT=x(iT,:);                                      // S15
yL=y(iL);            yT=y(iT);                                        // S16
dL=[xL yL];          dT=[xT yT];                                      // S17
cT=c(iT,:);                                                           // S18
//zT=z(iT);                                                           // S19
csvWrite(dL,'dLDiff.csv',';');     // write data to disk              // S20
csvWrite(dT,'dTDiff.csv',';');                                        // S21
disp('Data written to disk'), printf('\n')                            // S22
                                                                      // S23
nx=size(xL,2);                     // number of x-variables           // S24
ky=max(dL(:,$));                   // number of values in y           // S25
kx=max(dL(:,1:nx),'r');            // number of values in xi          // S26
                    
// INITIALIZATION                                                     // I1       
tic();                                                                // I2 
nd=length(yL);                     // data length for learning        // I3 
kI=10;                             // number of initial data          // I4 
pI=list();                         // initial parameters              // I5 
pI(1)=pS(1); pI(2)=pS(2);          // exact values (for validation)   // I6 
for i=1:nx, nc(i)=length(pI(i)); end                                  // I7 
                                                                      // I8 
for i=1:nx, for j=1:nc(i), N(j).x(i).k=kI; end, end // init.kappa     // I9 
for i=1:nx                                                            // I10
  for j=1:nc(i)                                                       // I11
    N(j).x(i).s=pI(i)(j)*N(j).x(i).k*(kx(i)-1)+N(j).x(i).k;           // I12
                                   // statistics for Binom.           // I13
  end                                                                 // I14
end                                                                   // I15
                                                                      // I16
for i=1:nx                                                            // I17
  for j=1:nc(i)                                                       // I18
    C(j).x(i).p=(N(j).x(i).s-N(j).x(i).k)/N(j).x(i).k/(kx(i)-1);      // I19
                                   // prior est. of parameters        // I20
  end                                                                 // I21
end                                                                   // I22
                                                                      
// ESTIMATION  of f(x) clusters                                       // E1 
for t=1:nd                                                            // E2 
  for i=1:nx        // for variables (i)                              // E3 
    q=0;                                                              // E4       
    for j=1:nc(i)   // for components (j)                             // E5 
      p=C(j).x(i).p;                                                  // E6 
      q(j)=binom1(xL(t,i),p,kx(i)); // prox. for clus. j in var i     // E7 
    end                                                               // E8 
    w=q/sum(q);                    // weights w for cl. j in var i    // E9 
    wt(i).w(:,t)=w;                // remember weights w              // E10
    for j=1:nc(i)                                                     // E11
      N(j).x(i).s=N(j).x(i).s +w(j)*xL(t,i); // update statistics     // E12
      N(j).x(i).k=N(j).x(i).k+w(j);                                   // E13
                                                                      // E14
      C(j).x(i).p=(N(j).x(i).s-N(j).x(i).k)/N(j).x(i).k/(kx(i)-1);    // E15
                                   // parameter estimates             // E16
      if C(j).x(i).p<0, C(j).x(i).p=0; end   // check for             // E17
      if C(j).x(i).p>1, C(j).x(i).p=1; end   //   probability         // E18
      pt(i,j)=C(j).x(i).p;         // remember estimates              // E19
    end,                                                              // E20
  end                                                                 // E21
  PT(:,t)=pt(:);                   // parameters for final display    // E22
end                                                                   // E23
                                                                      // E24
cp=list();                                                            // E25
for i=1:nx                                                            // E26
  cp(i)=amax(wt(i).w','c');        // estimated pointers              // E27      
end                                                                   // E28
Acc1=acc(c(iL,1),cp(1))                                               // E29
Acc2=acc(c(iL,2),cp(2))                                               // E30
                                                                      
// LOCAL MODELS f(xi|y) in clusters                                   // L1 
fy=pfY(yL,ky);                     // marginal f(y)                   // L2 
for i=1:nx                                                            // L3 
  for j=1:nc(i)                                                       // L4       
    k=find(cp(i)==j);              // k = set of times t              // L5 
                                   //   in cluster j and variable xi  // L6 
    f(j).x(i).c=pfXY(xL(k,i),yL(k),kx(i),ky);                         // L7 
                                   // conditional fj(xi|y)            // L8 
  end                                                                 // L9 
end                                                                   // L10
                                                                            
// PREDICTION                                                         // P1 
nd=length(yT);                     // data length for testing         // P2 
for t=1:nd                                                            // P3 
  Fyp=fy;                          // beginning of NB  f(y)           // P4 
  for i=1:nx                                                          // P5 
    q=0;                                                              // P6 
    for j=1:nc(i)                                                     // P7 
      p=C(j).x(i).p;                                                  // P8 
      q(j)=binom1(xT(t,i),p,kx(i)); // proximity pro xi v klastrech j // P9 
    end                                                               // P10
    w=q/sum(q);                    // weights for clusters j in xi    // P11
    WT(i).w(:,t)=w;                // remember weights                // P12
                                                                      // P13
    fpi=0;                         // formal start for summation      // P14
    for j=1:nc(i)                                                     // P15
      fpi=fpi+w(j)*f(j).x(i).c(xT(t,i),:);                            // P16
                                   // weighted sum for f(xi|y)        // P17      
    end                                                               // P18
    Fyp=Fyp.*fpi;                  // NB for  f(y|x)  over xi         // P19
  end                                                                 // P20
  yp(t)=amax(Fyp);                 // point estimate of y based on x  // P21
end                                                                   
                                                                      
// RESULTS                                                            
r=c2c(yT,yp);                                                         
disp 'Classification of y'                                                  
Acc=acc(yT(1:nd),r(yp))
disp 'Overall time'
toc(), printf('\n');
Vals_y=vals(yT)
Vals_yp=vals(r(yp))
