// ini4bCov.sce
// Fixní kovariance při odhadu směsi (odhad více komponent než v sim)
// ------------------------------------------
clc, clear, close(winsid()); mode(0)
[u,t,n]=file();                         // find working directory
chdir(dirname(n(2)));                   // set working directory
clear("u","t","n")                      // clear auxiliary data
getd c:\functions

nd=500;                  // number of data
nc=8;                    // number of componentd
t_estCov=50;             // time when est. of covars is switched on

// simulated reg.coef.
Sim.nc=nc;
Sim.Cy(1).th=[0.9 -.5]';
Sim.Cy(2).th=[0.4 0.6]';
// simulated noise covariances
Sim.Cy(1).cv=0.2*[1 -.6;-.6,1];
Sim.Cy(2).cv=0.1*[1 .2;.2,1];
// simulated pointer parameters
Sim.Cp.th=fnorm(ones(1,2)+.1,2);
Sim.ct(1)=1;                  // initial poiner

// SIMULATION ==========================================================
for t=2:nd
  Sim.ct(1,t)=sum(randu(1,1)>cumsum(Sim.Cp.th))+1;    // pointer
  j=Sim.ct(t); // active component
  y=Sim.Cy(j).th+uut(Sim.Cy(j).cv)*randn(2,1);        // output
  Sim.yt(:,t)=y;
end   

// initial parameters
my=mean(Sim.yt,2);
a=.3;                         // std of scattering initial parametrs
for j=1:nc                    //  from those used in simulation
  [mr,mc]=size(Sim.Cy(1).th);
  th=my+a*rand(mr,mc,'n');
  Ps=[th;1];                  // initial parameters
  Est.Cy(j).V=Ps*Ps';         // statistics
  Est.Cy(j).th=th;            // pt.est. of reg.coef
  Est.Cy(j).cv=.01*eye(2,2);  // standard deviation
end
Est.ka=ones(1,nc);            // counter
Est.Cp.V=ones(1,nc);          // pointer statistics
Est.Cp.th=fnorm(ones(1,nc));  // pointer parameter
w=fnorm(ones(1,nc));          // weights


// ESTIMATION ==========================================================
I_estCov=0;
printf(' ')
for t=2:nd  
  if t/fix(nd/10)==fix(t/fix(nd/10)), printf('.'); end
  for j=1:nc
    [xxx,G(j)]=GaussN(Sim.yt(:,t),Est.Cy(j).th,Est.Cy(j).cv); 
                                             // proximity
  end
  Lq=G-max(G);
  q=exp(Lq);

  ww=q'.*Est.Cp.th; w=ww/sum(ww);            // generation of weights

  // Update of statistic 
  Ps=[Sim.yt(:,t)' 1];                       // extended reg.vec.
  for i=1:nc
    Est.Cy(i).V=Est.Cy(i).V+w(i)*Ps'*Ps;     // information matrix
    Est.ka(i)=Est.ka(i)+w(i);                // counter
    Est.Cp.V(i)=Est.Cp.V(i)+w(i);            // pointer statistics

    //nove rozdeleni informacni matice V
    Vy=Est.Cy(i).V(1:2,1:2);                 // part Vy  - y.y
    Vyp=Est.Cy(i).V($,1:2);                  // part Vyp - psi.y
    Vp=Est.Cy(i).V($,$);                     // part Vp  - psi.psi'
    Est.Cy(i).th=inv(Vp+1e-8*eye(Vp))*Vyp;   // pt.est. - reg.coef.
    Est.Cy(i).tht(:,t)=Est.Cy(i).th';        // pt.est. - covar.
    if t>t_estCov, I_estCov=1; end
    if I_estCov~=0
      // PT.EST. OF NOISE COVARIANCE - USED OR NOT
      Est.Cy(i).cv=(Vy-Vyp'*inv(Vp+1e-8*eye(Vp))*Vyp)/Est.ka(i); 
    end
  end
  Est.Cp.th=fnorm(Est.Cp.V,2);     // pt.est. of pointer parameter
  [ss,Est.ct(1,t)]=max(w);         // store
end

s=2:nd;
Sc=Sim.ct(s);
Ec=Est.ct(s);
yt=Sim.yt(:,s);
S=list();                          // simulated clusters
for i=1:2
  j=find(Sc==i);
  S(i)=yt(:,j);
end  

C=list();                          // estimated clusters
for i=1:nc
  j=find(Ec==i);
  C(i)=yt(:,j);
end  

// Results
set(scf(1),'position',[300 100 1200 300])
for i=1:nc
subplot(1,nc,i)
plot(Est.Cy(i).tht(1,:),Est.Cy(i).tht(2,:),'.')
plot(Est.Cy(i).tht(1,2),Est.Cy(i).tht(2,2),'g.','markersize',18)
end
title 'green ring - beginning'

set(scf(2),'position',[300 550 1200 300])
subplot(1,nc+1,1)
plot(S(1)(1,:),S(1)(2,:),'b.')
plot(S(2)(1,:),S(2)(2,:),'r.')
set(gca(),'data_bounds',[-1 3 -1.5 2])
title 'Simulated'
for i=1:nc
subplot(1,nc+1,i+1)
if ~isempty(C(i))
plot(C(i)(1,:),C(i)(2,:),'.')
end
title 'Estimated'
set(gca(),'data_bounds',[-1 3 -1.5 2])
end








 
